@@ -8,8 +8,8 @@ Refs:
 
 """
 
-__version__ = "3.1.2"
+__version__ = "7.0.0"
 
 from .commons import BoundingBox, Coords, Tile  # noqa
-from .defaults import tms  # noqa
+from .defaults import TileMatrixSets, tms  # noqa
 from .models import TileMatrixSet  # noqa
