@@ -1,12 +1,13 @@
 """morecantile utils."""
 
 import math
-from typing import Dict, List
+from typing import Dict, List, Tuple
 
 from pyproj import CRS
+from pyproj.enums import WktVersion
 
-from .commons import BoundingBox, Coords, Tile
-from .errors import TileArgParsingError
+from morecantile.commons import BoundingBox, Coords, Tile
+from morecantile.errors import TileArgParsingError
 
 
 def _parse_tile_arg(*args) -> Tile:
@@ -39,6 +40,30 @@ def _parse_tile_arg(*args) -> Tile:
         )
 
 
+def lons_contain_antimeridian(lon1: float, lon2: float) -> bool:
+    """
+    Check if the antimeridian (180th meridian) is between two longitude points
+
+    Parameters
+    ----------
+    lon1: float
+        The first longitude.
+    lon2: float
+        The second longitude
+
+    Returns
+    -------
+    A bool representing whether two longs contain the 180th meridian.
+    """
+    lon1_clipped = max(-180.0, min(lon1, 180))
+    lon2_clipped = max(-180.0, min(lon2, 180))
+    lon1_converted = (lon1_clipped + 360) % 360
+    lon2_converted = (lon2_clipped + 360) % 360
+    ws = [lon1_converted, lon2_converted]
+    sorted(ws)
+    return ws[0] < 180 < ws[1]
+
+
 def meters_per_unit(crs: CRS) -> float:
     """
     Coefficient to convert the coordinate reference system (CRS)
@@ -52,17 +77,17 @@ def meters_per_unit(crs: CRS) -> float:
     """
     unit_factors = {
         "metre": 1.0,
-        "degree": 2 * math.pi * 6378137 / 360.0,
+        "degree": 2 * math.pi * crs.ellipsoid.semi_major_metre / 360.0,
         "foot": 0.3048,
         "US survey foot": 0.30480060960121924,
     }
     unit_name = crs.axis_info[0].unit_name
     try:
         return unit_factors[unit_name]
-    except KeyError:
+    except KeyError as e:
         raise Exception(
-            f"CRS {crs} is not supported, please fill an issue in developmentseed/morecantile"
-        )
+            f"CRS {crs} with Unit Name `{unit_name}` is not supported, please fill an issue in developmentseed/morecantile"
+        ) from e
 
 
 def bbox_to_feature(west: float, south: float, east: float, north: float) -> Dict:
@@ -85,6 +110,28 @@ def point_in_bbox(point: Coords, bbox: BoundingBox, precision: int = 5) -> bool:
     )
 
 
+def truncate_coordinates(
+    lng: float, lat: float, bbox: BoundingBox
+) -> Tuple[float, float]:
+    """
+    Truncate coordinates to a given bbox.
+
+    Adapted from https://github.com/mapbox/mercantile/blob/master/mercantile/__init__.py
+
+    """
+    if lng > bbox.right:
+        lng = bbox.right
+    elif lng < bbox.left:
+        lng = bbox.left
+
+    if lat > bbox.top:
+        lat = bbox.top
+    elif lat < bbox.bottom:
+        lat = bbox.bottom
+
+    return lng, lat
+
+
 def is_power_of_two(number: int) -> bool:
     """Check if a number is a power of 2"""
     return (number & (number - 1) == 0) and number != 0
@@ -93,10 +140,19 @@ def is_power_of_two(number: int) -> bool:
 def check_quadkey_support(tms: List) -> bool:
     """Check if a Tile Matrix Set supports quadkeys"""
     return all(
-        [
-            (t.matrixWidth == t.matrixHeight)
-            and is_power_of_two(t.matrixWidth)
-            and ((t.matrixWidth * 2) == tms[i + 1].matrixWidth)
-            for i, t in enumerate(tms[:-1])
-        ]
+        (t.matrixWidth == t.matrixHeight)
+        and is_power_of_two(t.matrixWidth)
+        and ((t.matrixWidth * 2) == tms[i + 1].matrixWidth)
+        for i, t in enumerate(tms[:-1])
     )
+
+
+def to_rasterio_crs(incrs: CRS):
+    """Convert a pyproj CRS to a rasterio CRS"""
+    from rasterio import crs
+    from rasterio.env import GDALVersion
+
+    if GDALVersion.runtime().major < 3:
+        return crs.CRS.from_wkt(incrs.to_wkt(WktVersion.WKT1_GDAL))
+    else:
+        return crs.CRS.from_wkt(incrs.to_wkt())
